module Revisions
  class Item < ApplicationRecord

    has_many :revisions, -> { order 'created_at desc' }, dependent: :destroy

    def add_revision(
      items_key_id: nil,
      duplicate_of: nil,
      content: nil,
      enc_item_key: nil,
      auth_hash: nil,
      auth_params: nil
    )
      revision = self.revisions.new
      revision.items_key_id = items_key_id
      revision.duplicate_of = duplicate_of
      revision.content = content
      revision.enc_item_key = enc_item_key
      revision.auth_hash = auth_hash
      revision.auth_params = auth_params.to_json
      revision.save
    end

    def prune_all_history
      prune_history(400)
    end

    # prunes recent history
    def prune_history(days = 30)
      # revisions past 30 days are limited to min_revs revision per day
      # any revision within 30 days is capped to 30-days_from_today revisions per day
      # so if there are 50 revisions from 5 days ago, 50 - (30-5) = 25 revisions are allowed
      max_revs = 8
      min_revs = 4
      days.times do |days_from_today|
        date = Time.now.utc.to_date - days_from_today
        num_revs_target = [days - days_from_today + min_revs, max_revs].min # actual number kept may be a little more or less
        revs = self.revisions.where(:created_at => date.midnight..date.end_of_day).pluck(:id)
        if revs.length > num_revs_target
          keep_every_nth = (revs.length.to_f / (num_revs_target)).ceil
          keep = [revs.length - 1]

          (0..revs.length - keep_every_nth).step(keep_every_nth) do |n|
            keep.push(n)
          end

          discard_ids = []
          revs.length.times do |index|
            if keep.include?(index) == false
              discard_ids.push(revs[index])
            end
          end

          Revision
            .where(id: discard_ids)
            .delete_all
        end
      end
    end

  end
end
